function [s,dual_values,primal_values,gaps,bundle,added1,added2,added3] = minimize_submodular_FW_minnormpoint_proxabs(F,param_F,z,maxiter,gap,bundle)
% Compute  min_x .5 * || s - z ||^2 such that s in |P|(F) using Frank-Wolfe on the dual (using
% the min norm point algorithm)
%
%
%
% INPUT
% F,param_F: submodular function
% maxiter: maximum number of iterations
%
% OUTPUT
% x: argmin. If sfm=1, then outputs the subset
% values of cost function
% gaps: certified optimaly gaps at each iteration

if nargout>=8
    tic;
end

n = param_F.p;

display = 1;


if nargin < 6
    % use random initialization
    direction = rand(n,1);
    
    x = -z + sign(direction) .* greedy_algo_submodular(abs(direction),F,param_F);
    w = 1;
    X = x;
    
else
    x = -z + bundle.x;
    X = bundle.X - repmat(z,1,size(bundle.X,2));
    w = bundle.w;
end
nstep=1;
iter = 0;
bestvalue = Inf;
dobundle=1;

while iter < maxiter
    iter = iter + 1;
    
    if nargout>=8
        added3(iter) = toc;
    end
    
    
    % step 1
    % a
    x = X*w;
    
    direction =  - x;
    
    % b
    [xx,Fvalues,order] =  greedy_algo_submodular(abs(direction),F,param_F);
    xx = -z + xx .* sign(direction) ;
    % c
    
    
    
    
    
    
    % compute values of the function
    dual_values(iter) = .5*z'*z - .5 * x' * x;
    temp = abs(z) - abs(xx+z);
    ww = pav(   temp(flipud(order)) );
    ww(flipud(order)) = max(ww,0);
    ww = ww .* sign(z);
    
    
    added1(iter)  =  (xx+z)'*(-x) +.5*(x+z)'*(x+z); % compute value without PAV
    primal_values(iter)  = .5*(ww-z)'*(ww-z) + ww'*(xx+z);
    
    gaps(iter) = primal_values(iter) - dual_values(iter);
    added2(iter)  = added1(iter) - dual_values(iter); % compute gap without PAV
    if gaps(iter) < gap
        % stop!
        if display, fprintf('stopped at step 1c - reached upper bound on duality gap \n'); end
        break;
    end
    
    
    
    
    % d: do not check at the first iteration (because of restarts)
    if iter>1 && min(sum( (X - repmat(xx,1,size(X,2))).^2 , 1 ) ) < 1e-12 *max(sum(X.^2,1))
        % stop!
        iter
        if display, fprintf('stopped at step 1d\n'); end
        dobundle=0;
        break;
    end
    
    % e
    X = [X, xx];
    w = [w; 0];
    
    iterloc =1;
    while 1,
        iterloc = iterloc + 1;
        if iterloc > 100*n,
            fprintf('probably looping between step 2 and 3, exit \n');
            iterloc = 0;
            dobundle=0;
            break;
        end
        % step 2
        % a
        try
            R = chol( X'*X + 1/size(X,2) * norm(X,'fro').^2 + 1e-13 * 1/size(X,2) * norm(X,'fro').^2 * eye( size(X,2)) );
        catch
            % not positive definite
            iterloc = 0;
            fprintf('not positive definite when adding new point in step 2, exit\n');
            dobundle=0;
            
            break;
        end
        v = R \ ( R' \ ones(size(X,2),1) );
        v = v / sum(v);
        
        % checking
        % X'* ( X * v );
        
        % b
        if all( v > 1e-12),
            w = v;
            break;
            % go to step 1
        else
            
            % step 3
            % a
            ind = find( w - v > 1e-12);
            
            if isempty(ind)
                iterloc = 0;
                fprintf('can''t do line seaerch in step 2, exit\n');
                dobundle=0;
                
                break;
            end
            %b
            theta = min( 1, min ( w(ind)./(w(ind)-v(ind))));
            %c
            w = theta * v + (1-theta) * w;
            %d-e
            torem = find( w < 1e-12);
            w(torem) = [];
            X(:,torem) = [];
            w = w / sum(w);
            % go to step 2
        end
    end
    if iterloc==0, break; end
end
if nargout>=8
    added3(iter+1) = toc;
end

s = x + z;

% save bundle
if dobundle
    bundle.x=x+z;
    bundle.X=X+repmat(z,1,size(X,2));
    bundle.w = w;
else
    bundle.x=x+z;
    bundle.X=x;
    bundle.w = 1;
    
    
end

